#include <libopencm3/stm32/f1/memorymap.h>
#include <libopencm3/stm32/f4/scb.h>
#include <libopencm3/stm32/f4/rcc.h>
#include <libopencm3/stm32/f4/flash.h>
#include <libopencm3/stm32/f4/pwr.h>
#include "System.h"

// PLL_VCO = (HSE_VALUE or HSI_VALUE / PLL_M) * PLL_N
#define PLL_M 8
#define PLL_N 336

// SYSCLK = PLL_VCO / PLL_P
#define PLL_P 2

// USB OTG FS, SDIO and RNG Clock =  PLL_VCO / PLLQ
#define PLL_Q 7

// PLLI2S_VCO = (HSE_VALUE Or HSI_VALUE / PLL_M) * PLLI2S_N
// I2SCLK = PLLI2S_VCO / PLLI2S_R
#define PLLI2S_N 192
#define PLLI2S_R 5

void InitializeSystem()
{
	// Reset the RCC clock configuration to the default reset state
	RCC_CR |= 0x00000001; // Set HSION bit
	RCC_CFGR = 0x00000000;	// Reset CFGR register
	RCC_CR &= 0xFEF6FFFF; // Reset HSEON, CSSON and PLLON bits
	RCC_PLLCFGR = 0x24003010; // Reset PLLCFGR register
	RCC_CR &= 0xFFFBFFFF; // Reset HSEBYP bit
	RCC_CIR = 0x00000000; // Disable all interrupts

	// Configure the System clock source, PLL Multiplier and Divider factors, 
	// AHB/APBx prescalers and Flash settings
	rcc_clock_setup_hse_3v3(&hse_8mhz_3v3[CLOCK_3V3_168MHZ]);

	// Set vector table offset to flash memory start.
	SCB_VTOR = FLASH_BASE;

	// Set up interrupts to 4 bits preemption priority.
	SCB_AIRCR = 0x05FA0000|0x300;
}

// Interrupt handlers.

static InterruptHandler **WritableInterruptTable();
void Default_Handler() { };

extern InterruptHandler *__isr_vector_sram[];

void InstallInterruptHandler(IRQn_Type interrupt,InterruptHandler handler)
{
	// TODO: Disable interrupts.
	InterruptHandler **currenttable = WritableInterruptTable();
	currenttable[interrupt+16] = handler;
}

void RemoveInterruptHandler(IRQn_Type interrupt,InterruptHandler handler)
{
	// TODO: Disable interrupts.
	InterruptHandler **currenttable = WritableInterruptTable();
	currenttable[interrupt+16]=Default_Handler;
}

static InterruptHandler **WritableInterruptTable()
{
	InterruptHandler **currenttable=(InterruptHandler **)SCB_VTOR;
	if ((uint32_t)currenttable == FLASH_BASE) {
		int i;
		InterruptHandler **flashtable = (InterruptHandler **)FLASH_BASE;
		currenttable = __isr_vector_sram;
		for (i = 0; i < 98; i++) currenttable[i] = flashtable[i];

		SCB_VTOR = (uint32_t)currenttable;
	}
	return currenttable;
}
